"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convert = void 0;
var fs = __importStar(require("fs"));
var camelize_1 = require("./helpers/camelize");
var handleKeyword = function (keyword, content) {
    switch (keyword) {
        case 'keys': {
            var keys = content.split(',');
            return keys.map(function (key) { return (0, camelize_1.camelize)(key.trim()); });
        }
        case 'name': {
            return (0, camelize_1.camelize)(content);
        }
        default: {
            console.error("Keyword ".concat(keyword, " unrecognized."));
            throw new Error();
        }
    }
};
var convertMetadata = function (metadata) {
    var acc = { keys: [] };
    return metadata.reduce(function (acc, val) {
        var _a;
        var _b = val.split(':'), keyword = _b[0], value = _b[1];
        return __assign(__assign({}, acc), (_a = {}, _a[keyword] = handleKeyword(keyword, value.trim()), _a));
    }, acc);
};
var joinCommand = function (command) {
    var comm = command.join(' ');
    if (comm.endsWith(';')) {
        return comm.slice(0, -1);
    }
    else {
        return comm;
    }
};
var convertCommand = function (_a, command) {
    var keys = _a.keys;
    var joinedCommand = joinCommand(command);
    if (keys) {
        return keys.reduce(function (comm, key, index) {
            var matcher = new RegExp("\\$".concat(key), 'g');
            return comm.replace(matcher, "$".concat(index + 1));
        }, joinedCommand);
    }
    else {
        return joinedCommand;
    }
};
var toCorrectQuery = function (command) {
    return command
        .replace(/\s\s+/g, ' ')
        .replace(/\(\s+/g, '(')
        .replace(/\s+\)/g, ')');
};
var newCommand = function (metadata, command) {
    var meta = convertMetadata(metadata);
    return {
        metadata: meta,
        pp: command.join('\n'),
        command: toCorrectQuery(convertCommand(meta, command)),
    };
};
var keepMetadata = function (_a, line) {
    var metadata = _a[0], command = _a[1];
    if (line.trim().startsWith('--')) {
        if (command.length === 0) {
            var allMetadata = __spreadArray(__spreadArray([], metadata, true), [line.slice(2).trim()], false);
            return [allMetadata, command];
        }
        else {
            return [metadata, command];
        }
    }
    else {
        var allCommand = __spreadArray(__spreadArray([], command, true), [line], false);
        return [metadata, allCommand];
    }
};
var toCommand = function (lines) {
    var accumulator = [[], []];
    var _a = lines.reduce(keepMetadata, accumulator), metadata = _a[0], command = _a[1];
    return newCommand(metadata, command);
};
var groupByCommand = function (_a, line) {
    var commands = _a.commands, lines = _a.lines;
    var onlyCode = line.trim();
    if (onlyCode.startsWith('--')) {
        return { commands: commands, lines: __spreadArray(__spreadArray([], lines, true), [line], false) };
    }
    else if (onlyCode.endsWith(';')) {
        return { commands: __spreadArray(__spreadArray([], commands, true), [__spreadArray(__spreadArray([], lines, true), [line], false)], false), lines: [] };
    }
    else if (onlyCode.length === 0) {
        return { commands: commands, lines: lines };
    }
    else {
        return { commands: commands, lines: __spreadArray(__spreadArray([], lines, true), [line], false) };
    }
};
var extractCommands = function (file) {
    var splitted = file.split('\n');
    var accumulator = { commands: [], lines: [] };
    var _a = splitted.reduce(groupByCommand, accumulator), commands = _a.commands, lines = _a.lines;
    if (lines.length === 0) {
        return commands;
    }
    else {
        return __spreadArray(__spreadArray([], commands, true), [lines], false);
    }
};
var createFunction = function (client, keys, command, pp) {
    var fun = function (args) {
        var argsValue = keys.map(function (key) { return args[(0, camelize_1.camelize)(key)]; });
        return client.query(command, argsValue);
    };
    fun.toString = function () { return pp; };
    fun.command = function () { return command; };
    return fun;
};
var turnToFunction = function (client, commands) {
    return commands.reduce(function (functions, command_) {
        var _a;
        var metadata = command_.metadata, command = command_.command, pp = command_.pp;
        var name = metadata.name, keys = metadata.keys;
        if (!name)
            return functions;
        var func = createFunction(client, keys, command, pp);
        return __assign(__assign({}, functions), (_a = {}, _a[name] = func, _a));
    }, {});
};
var convert = function (client) { return function (filePath) {
    var file = fs.readFileSync(filePath, 'utf8');
    var rawCommands = extractCommands(file);
    var commands = rawCommands.map(toCommand);
    var functions = turnToFunction(client, commands);
    return functions;
}; };
exports.convert = convert;
//# sourceMappingURL=index.js.map